/**********************************************************************************
	Bootloader for AT91SAM7S

	2006.6.11 jsy (hwimh@hanmail.net)
**********************************************************************************/
#include "at91sam7s64.h"
#include "uart.h"

#include "u_boot256.h"			// u_boot for AT91SAM7S256/128
//#include "u_boot64.h"			// u_boot for AT91SAM7S64


#define SECTIME					0x49200 / 100		// 1msec
#define DELAYTIME				500					// Wait for 'H' until this time in msec

typedef void (*Func_Ptr)(void);


void delay_500ms(void)
{
	unsigned long	Time;

	Time = *AT91C_PITC_PIIR;
	while((*AT91C_PITC_PIIR - Time) < (DELAYTIME * SECTIME))
	{
		if(UART_IsRxReady())
			return;
	}
}


// don't use memcpy because it will addet to flash!
void slowmemcpy(void* dst, void* src, unsigned int length)
{
	char* cdst = (char*)dst;
	char* csrc = (char*)src;
	for(unsigned int i=0; i<length; i++)
	{
		*cdst = *csrc;
		cdst++;
		csrc++;
	}
}


void Call_u_Boot(void)
{
	Func_Ptr	u_boot_fn;

	//copy u-boot to RAM
	slowmemcpy((unsigned int*) U_BOOT_ADDRESS, (char*)u_boot_image, U_BOOT_LENGTH);

	//Run the u-boot
	u_boot_fn = (Func_Ptr)U_BOOT_ADDRESS;
	(*u_boot_fn)();
}


void uart_loop(void)
{
	UART_Write("X", 1);
	delay_500ms();

	if(UART_IsRxReady())
	{
		char	buf[4];

		// sync with PC
		UART_Read((char*)buf, 1);
		if(buf[0] != 'H')	// wait Hello!
			return;

		*AT91C_PITC_PIMR = 0x00000000;		// stop PIT

		Call_u_Boot();
	}
}


void BootLoader_Do(void)
{
	unsigned int	boottimeout = 0;

	while(boottimeout < 2*3)		// 3 sec
	{
		uart_loop();
		boottimeout++;
	}
}


void StartBootLoader(void)
{
	UART_Init();

	// init Periodic Interval timer
	*AT91C_PITC_PIMR = 0x000fffff | AT91C_PITC_PITEN;

	BootLoader_Do();

	//---------------------------------------------------------
	// below lines only executed when bootloader timeout
	//---------------------------------------------------------
	*AT91C_PITC_PIMR = 0x00000000;		// stop PIT
}

